//
// Created by philipp on 20.05.18.
//

#include "../include/PlatoonProtocolLib/VehicleFacade.h"
#include "../include/PlatoonProtocolLib/FollowerVehicle.h"

int main(int argc, char ** argv)
{
    using namespace platoonProtocol;

    if (argc < 2)
    {
        std::cerr << "Too few arguments!\n";
        return -1;
    }

    auto ownVehicleId = static_cast<VehicleId>(std::atoi(argv[1]));
    Vehicle::VehicleEndpoints endpoints;

    for (int arg = 2; arg + 1 < argc; arg += 2)
    {
        auto vehicleId = static_cast<VehicleId>(std::atoi(argv[arg]));
        std::string host{argv[arg + 1]};
        endpoints.emplace_back(VehicleEndpoint{vehicleId, host});
    }

    networking::Networking net;
    std::atomic<bool> running{false};
    VehicleFacade follower{Vehicle::Role::FOLLOWER, net, ownVehicleId};
    follower.addVehicleEndpoints(endpoints);

    auto followerPtr = std::static_pointer_cast<FollowerVehicle>(follower.get());

    followerPtr->setOnRunningPlatoonCallback(
        [&running]
        {
            running = true;
            std::cout << "Platoon is running!\n";
        });

    followerPtr->setOnLeavingPlatoonCallback(
        [&running]
        {
            running = false;
            std::cout << "Left platoon!\n";
        });

    followerPtr->setOnPlatoonConfigUpdatedCallback(
        [&]
        {
            auto config = follower.getPlatoonConfig();
            std::cout << "Config Update:\nIPD=" << config.get().innerPlatoonDistance
                      << "\nPS=" << config.get().platoonSpeed << "\n";
        });

    follower.createPlatoon();

    while (1)
    {
        std::string input;
        std::cin >> input;
        if (input == "q")
            break;
    }

    follower.leavePlatoon();
    sleep(1);

    return 0;
}